/***************************************************************************
           dialogexportwav.cpp  -  Dialog for setting WAV parameters
          -----------------------------------------------------------
    begin                : Fri Mar 16 2001
    copyright            : (C) 2001 by Ladislav Slezk
    email                : slezak@penguin.cz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "dialogexportwav.h"

#include <libgnome/gnome-i18n.h>
#include <gnome--/entry.h>

#include <list>
#include <iostream.h>

using namespace std;

DialogExportWAV::DialogExportWAV() : Gtk::Window(GTK_WINDOW_DIALOG) {
  set_modal(true);
  set_border_width(10);
  set_title(_("Export as WAV file"));
  set_policy(false, false, false);    // set fixed size of window

  string history;
  fentry = manage(new Gnome::FileEntry(history, _("Select WAV file")));
  fentry->set_modal(true);
  fentry->set_border_width(5);
  fentry->set_default_path("*.wav");

  Gnome::Entry *gentry = dynamic_cast<Gnome::Entry *>(fentry->gnome_entry());
  gentry->set_history_id("WAV_export_history");
  gentry->load_history();

  Gtk::VBox *vbox1 = manage(new Gtk::VBox(false, 5));

  Gtk::Frame *fbox1 = manage(new Gtk::Frame(_("WAV file")));
  Gtk::Frame *fbox2 = manage(new Gtk::Frame(_("Properties")));

  vbox1->pack_start(*fbox1, false, false, 5);
  vbox1->pack_start(*fbox2, false, false, 5);
  Gtk::VBox *vbox = manage(new Gtk::VBox(false, 5));
  vbox->pack_start(*fentry, false, false, 5);
  fbox1->add(*vbox);

  Gtk::VBox *vbox2 = manage(new Gtk::VBox(false, 3));
  vbox2->set_border_width(3);

  Gtk::HBox *hbox1 = manage(new Gtk::HBox(false, 5));
  Gtk::Label *l1 = manage(new Gtk::Label(_("Number of channels ")));

  opt_chan = manage(new Gtk::OptionMenu);
  Gtk::Menu *menu_chan = manage(new Gtk::Menu);
  Gtk::Menu_Helpers::MenuList &menu_chan_list = menu_chan->items();
  menu_chan_list.push_back(Gtk::Menu_Helpers::MenuElem(_("1 - Mono"), bind(slot(this, &DialogExportWAV::chan_changed), 1)));
  menu_chan_list.push_back(Gtk::Menu_Helpers::MenuElem(_("2 - Stereo"), bind(slot(this, &DialogExportWAV::chan_changed), 2)));
  chan = 1;
  menu_chan->set_active(0);
  opt_chan->set_menu(menu_chan);

  hbox1->pack_start(*l1, false, false, 5);
  hbox1->pack_end(*opt_chan, false, false, 5);
  vbox2->pack_start(*hbox1, false, false);


  Gtk::HBox *hbox2 = manage(new Gtk::HBox(false, 5));
  Gtk::Label *l2 = manage(new Gtk::Label(_("Sampling frequency ")));

  opt_freq = manage(new Gtk::OptionMenu);
  Gtk::Menu *menu_freq = manage(new Gtk::Menu);
  Gtk::Menu_Helpers::MenuList &menu_freq_list = menu_freq->items();
  menu_freq_list.push_back(Gtk::Menu_Helpers::MenuElem(_("8000 Hz"), bind(slot(this, &DialogExportWAV::freq_changed), 8000)));
  menu_freq_list.push_back(Gtk::Menu_Helpers::MenuElem(_("11025 Hz"), bind(slot(this, &DialogExportWAV::freq_changed), 11025)));
  menu_freq_list.push_back(Gtk::Menu_Helpers::MenuElem(_("16000 Hz"), bind(slot(this, &DialogExportWAV::freq_changed), 16000)));
  menu_freq_list.push_back(Gtk::Menu_Helpers::MenuElem(_("22050 Hz"), bind(slot(this, &DialogExportWAV::freq_changed), 22050)));
  menu_freq_list.push_back(Gtk::Menu_Helpers::MenuElem(_("24000 Hz"), bind(slot(this, &DialogExportWAV::freq_changed), 24000)));
  menu_freq_list.push_back(Gtk::Menu_Helpers::MenuElem(_("32000 Hz"), bind(slot(this, &DialogExportWAV::freq_changed), 32000)));
  menu_freq_list.push_back(Gtk::Menu_Helpers::MenuElem(_("44100 Hz"), bind(slot(this, &DialogExportWAV::freq_changed), 44100)));
  menu_freq_list.push_back(Gtk::Menu_Helpers::MenuElem(_("48000 Hz"), bind(slot(this, &DialogExportWAV::freq_changed), 48000)));
  menu_freq->set_active(0);
  freq = 8000;
  opt_freq->set_menu(menu_freq);

  hbox2->pack_start(*l2, false, false, 5);
  hbox2->pack_end(*opt_freq, false, false, 5);
  vbox2->pack_start(*hbox2, false, false);


  Gtk::HBox *hbox3 = manage(new Gtk::HBox(false, 5));
  Gtk::Label *l3 = manage(new Gtk::Label(_("Bits per sample ")));

  opt_bps = manage(new Gtk::OptionMenu);
  Gtk::Menu *menu_bps = manage(new Gtk::Menu);
  Gtk::Menu_Helpers::MenuList &menu_bps_list = menu_bps->items();
  menu_bps_list.push_back(Gtk::Menu_Helpers::MenuElem(_("8 bits/sample"), bind(slot(this, &DialogExportWAV::bps_changed), 8)));
  menu_bps_list.push_back(Gtk::Menu_Helpers::MenuElem(_("16 bits/sample"), bind(slot(this, &DialogExportWAV::bps_changed), 16)));
  bps = 8;
  menu_bps->set_active(0);
  opt_bps->set_menu(menu_bps);

  hbox3->pack_start(*l3, false, false, 5);
  hbox3->pack_end(*opt_bps, false, false, 5);
  vbox2->pack_start(*hbox3, false, false);


  fbox2->add(*vbox2);

  Gtk::HButtonBox *hbbox1 = manage(new Gtk::HButtonBox());
  vbox1->pack_start(*hbbox1, true, false, 0);

//  Gnome::Stock::Buttons::Ok *bok = manage(new Gnome::Stock::Buttons::Ok);
  Gnome::StockButton *bok = manage(new Gnome::StockButton(GNOME_STOCK_BUTTON_OK));

  bok->clicked.connect(bind(slot(this, &DialogExportWAV::button_pressed), true));

  Gnome::StockButton *bcancel = manage(new Gnome::StockButton(GNOME_STOCK_BUTTON_CANCEL));
  bcancel->clicked.connect(bind(slot(this, &DialogExportWAV::button_pressed), false));

  hbbox1->add(*bok);
  hbbox1->add(*bcancel);
  hbbox1->set_layout(GTK_BUTTONBOX_SPREAD);

  add(*vbox1);

  show_all();
}

DialogExportWAV::~DialogExportWAV(){

}

void DialogExportWAV::button_pressed(bool ok) {
  // save history
  Gnome::Entry *gentry = dynamic_cast<Gnome::Entry *>(fentry->gnome_entry());
  if (gentry) gentry->save_history();

  export_ack.emit(ok);
  hide();
}


string DialogExportWAV::get_filename() {
  // method gnome_entry returns Gtk::Widget *, dynamic cast is required
  Gnome::Entry *gentry = dynamic_cast<Gnome::Entry *>(fentry->gnome_entry());

  if (gentry)
    return gentry->get_entry()->get_text();
  else
    return string("");
}

void DialogExportWAV::freq_changed(int f) {
#ifdef DEBUG_OUTPUT
  cerr << "freq = " << f << endl;
#endif

  freq = f;
}

void DialogExportWAV::bps_changed(int b) {
#ifdef DEBUG_OUTPUT
  cerr << "bps = " << b << endl;
#endif

  bps = b;
}

void DialogExportWAV::chan_changed(int c) {
#ifdef DEBUG_OUTPUT
  cerr << "ch = " << c << endl;
#endif

  chan = c;
}

int DialogExportWAV::delete_event_impl(GdkEventAny* e) {
  hide();

  return true;            // disable destroying window by WM
}

void DialogExportWAV::set_bps(int b) {
  switch (b) {
    case (8)  : opt_bps->set_history(0); bps = 8; break;
    case (16) : opt_bps->set_history(1); bps = 16; break;
    default : break;
  }
}

void DialogExportWAV::set_frequency(int f) {
  int old_f = freq;

  freq = f;

  switch (f) {
    case (8000)  : opt_freq->set_history(0); break;
    case (11025) : opt_freq->set_history(1); break;
    case (16000) : opt_freq->set_history(2); break;
    case (22050) : opt_freq->set_history(3); break;
    case (24000) : opt_freq->set_history(4); break;
    case (32000) : opt_freq->set_history(5); break;
    case (44100) : opt_freq->set_history(6); break;
    case (48000) : opt_freq->set_history(7); break;
    default : freq = old_f; break;
  }
}

void DialogExportWAV::set_channels(int ch) {
  switch (ch) {
    case (1) : opt_chan->set_history(0); chan = 1; break;
    case (2) : opt_chan->set_history(1); chan = 2; break;
    default : break;
  }
}

