/***************************************************************************
     dialoglameoptions.cpp  -  Dialog for setting specific Lame options
    --------------------------------------------------------------------
    begin                : Sat Mar 31 2001
    copyright            : (C) 2001 by Ladislav Slezk
    email                : slezak@penguin.cz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "dialoglameoptions.h"

#include <libgnome/gnome-i18n.h>
#include <gnome--/entry.h>


DialogLameOptions::DialogLameOptions() : Gtk::Window(GTK_WINDOW_DIALOG) {
  set_modal(true);
  set_title(_("Lame encoder options"));
  set_policy(false, false, false);    // set fixed size of window

  Gtk::Tooltips *tips = manage(new Gtk::Tooltips);

  bit_reservoir = manage(new Gtk::CheckButton(_("Disable bit reservoir"), 0.0));
  bit_reservoir->set_border_width(3);
  bit_reservoir->set_active(true);

  Gtk::Frame *frame_stream = manage(new Gtk::Frame(_("Stream options")));
  frame_stream->set_border_width(3);
  frame_stream->add(*bit_reservoir);


  q_high = manage(new Gtk::RadioButton(_("High"), 0.0));
  q_normal = manage(new Gtk::RadioButton(_("Normal"), 0.0));
  q_low = manage(new Gtk::RadioButton(_("Low"), 0.0));
  // set equal groups
  q_normal->set_group(q_high->group());
  q_low->set_group(q_high->group());

  // set active 'normal' quality
  q_normal->set_active(true);

  Gtk::VBox *vbox_q = manage(new Gtk::VBox(false, 3));
  vbox_q->pack_start(*q_high, true, true);
  vbox_q->pack_start(*q_normal, true, true);
  vbox_q->pack_start(*q_low, true, true);
  vbox_q->set_border_width(3);

  Gtk::Frame *frame_quality = manage(new Gtk::Frame(_("CBR encoding quality")));
  frame_quality->set_border_width(3);
  frame_quality->add(*vbox_q);


  filter_all_freqs = manage(new Gtk::CheckButton(_("Keep all frequencies (disable all filters)"), 0.0));
  filter_all_freqs->clicked.connect(slot(this, &DialogLameOptions::all_frequencies));

  filter_lowpass_cutoff = manage(new Gtk::CheckButton(_("Lowpass filter cutoff above"), 0.0));
  filter_lowpass_cutoff->clicked.connect(slot(this, &DialogLameOptions::low_cutoff));
  filter_lowpass_cutoff_entry = manage(new Gtk::Entry(6));
  filter_lowpass_cutoff_entry->set_usize(60, -1);
  filter_lowpass_cutoff_entry->set_sensitive(false);
  low_kHz = manage(new Gtk::Label(_("kHz"), 0.0));
  low_kHz->set_sensitive(false);
  Gtk::HBox *hbox_lowpass_cutoff = manage(new Gtk::HBox(false, 3));
  hbox_lowpass_cutoff->pack_start(*filter_lowpass_cutoff);
  hbox_lowpass_cutoff->pack_end(*low_kHz, false, false);
  hbox_lowpass_cutoff->pack_end(*filter_lowpass_cutoff_entry, false, false);

  filter_lowpass_width = manage(new Gtk::CheckButton(_("Lowpass filter width"), 0.0));
  filter_lowpass_width->clicked.connect(slot(this, &DialogLameOptions::low_width));
  filter_lowpass_width_entry = manage(new Gtk::Entry(6));
  tips->set_tip(*filter_lowpass_width_entry, _("Default value is 15% of lowpass frequency"));
  filter_lowpass_width_entry->set_usize(60, -1);
  filter_lowpass_width_entry->set_sensitive(false);
  low_w_kHz = manage(new Gtk::Label(_("kHz"), 0.0));
  low_w_kHz->set_sensitive(false);
  Gtk::HBox *hbox_lowpass_width = manage(new Gtk::HBox(false, 3));
  hbox_lowpass_width->pack_start(*filter_lowpass_width);
  hbox_lowpass_width->pack_end(*low_w_kHz, false, false);
  hbox_lowpass_width->pack_end(*filter_lowpass_width_entry, false, false);

  filter_highpass_cutoff = manage(new Gtk::CheckButton(_("Highpass filter cutoff above"), 0.0));
  filter_highpass_cutoff->clicked.connect(slot(this, &DialogLameOptions::high_cutoff));
  filter_highpass_cutoff_entry = manage(new Gtk::Entry(6));
  filter_highpass_cutoff_entry->set_usize(60, -1);
  filter_highpass_cutoff_entry->set_sensitive(false);
  high_kHz = manage(new Gtk::Label(_("kHz"), 0.0));
  high_kHz->set_sensitive(false);
  Gtk::HBox *hbox_highpass_cutoff = manage(new Gtk::HBox(false, 3));
  hbox_highpass_cutoff->pack_start(*filter_highpass_cutoff);
  hbox_highpass_cutoff->pack_end(*high_kHz, false, false);
  hbox_highpass_cutoff->pack_end(*filter_highpass_cutoff_entry, false, false);

  filter_highpass_width = manage(new Gtk::CheckButton(_("Highpass filter width"), 0.0));
  filter_highpass_width->clicked.connect(slot(this, &DialogLameOptions::high_width));
  filter_highpass_width_entry = manage(new Gtk::Entry(6));
  tips->set_tip(*filter_highpass_width_entry, _("Default value is 15% of highpass frequency"));
  filter_highpass_width_entry->set_usize(60, -1);
  filter_highpass_width_entry->set_sensitive(false);
  high_w_kHz = manage(new Gtk::Label(_("kHz"), 0.0));
  high_w_kHz->set_sensitive(false);
  Gtk::HBox *hbox_highpass_width = manage(new Gtk::HBox(false, 3));
  hbox_highpass_width->pack_start(*filter_highpass_width);
  hbox_highpass_width->pack_end(*high_w_kHz, false, false);
  hbox_highpass_width->pack_end(*filter_highpass_width_entry, false, false);

  filter_tonality = manage(new Gtk::CheckButton(_("Compute tonality up to"), 0.0));
  filter_tonality->clicked.connect(slot(this, &DialogLameOptions::tonality));
  filter_tonality_entry = manage(new Gtk::Entry(6));
  filter_tonality_entry->set_usize(60, -1);
  filter_tonality_entry->set_sensitive(false);
  tips->set_tip(*filter_tonality_entry, _("Default frequency is 8.8717 kHz"));
  tonality_kHz = manage(new Gtk::Label(_("kHz"), 0.0));
  tonality_kHz->set_sensitive(false);
  Gtk::HBox *hbox_tonality = manage(new Gtk::HBox(false, 3));
  hbox_tonality->pack_start(*filter_tonality);
  hbox_tonality->pack_end(*tonality_kHz, false, false);
  hbox_tonality->pack_end(*filter_tonality_entry, false, false);

  Gtk::VBox *vbox_f = manage(new Gtk::VBox(false, 3));
  vbox_f->pack_start(*filter_all_freqs, true, true);
  vbox_f->pack_start(*hbox_lowpass_cutoff, true, true);
  vbox_f->pack_start(*hbox_lowpass_width, true, true);
  vbox_f->pack_start(*hbox_highpass_cutoff, true, true);
  vbox_f->pack_start(*hbox_highpass_width, true, true);
  vbox_f->pack_start(*hbox_tonality, true, true);
  vbox_f->set_border_width(3);

  Gtk::Frame *frame_filter = manage(new Gtk::Frame(_("Filter options")));
  frame_filter->set_border_width(3);
  frame_filter->add(*vbox_f);


  Gnome::StockButton *button_ok = manage(new Gnome::StockButton(GNOME_STOCK_BUTTON_OK));
  button_ok->clicked.connect(slot(this, &DialogLameOptions::button_pressed));
//  Gnome::StockButton *button_cancel = manage(new Gnome::StockButton(GNOME_STOCK_BUTTON_CANCEL));
//  button_cancel->clicked.connect(bind(slot(this, &DialogExportMP3::button_pressed), false));

  Gtk::HButtonBox *hbuttonbox = manage(new Gtk::HButtonBox(GTK_BUTTONBOX_SPREAD, 5));
  hbuttonbox->add(*button_ok);
//  hbuttonbox->add(*button_cancel);
  hbuttonbox->set_border_width(5);

  Gtk::VBox *vbox = manage(new Gtk::VBox(false, 3));
  vbox->pack_start(*frame_stream, false, false);
  vbox->pack_start(*frame_quality, false, false);
  vbox->pack_start(*frame_filter, false, false);
  vbox->pack_start(*hbuttonbox, false, false);
  vbox->set_border_width(5);
  add(*vbox);

}

DialogLameOptions::~DialogLameOptions() {

}

void DialogLameOptions::button_pressed() {
  hide();
}

void DialogLameOptions::all_frequencies() {
  bool state = !filter_all_freqs->get_active();

  filter_lowpass_cutoff->set_sensitive(state);
  filter_highpass_cutoff->set_sensitive(state);
  filter_lowpass_width->set_sensitive(state);
  filter_highpass_width->set_sensitive(state);

  if (state) {        // filter is enabled
    low_cutoff();
    high_cutoff();
    low_width();
    high_width();
  }
  else {
    filter_lowpass_cutoff_entry->set_sensitive(state);
    low_kHz->set_sensitive(state);
    filter_lowpass_width_entry->set_sensitive(state);
    low_w_kHz->set_sensitive(state);
    filter_highpass_cutoff_entry->set_sensitive(state);
    high_kHz->set_sensitive(state);
    filter_highpass_width_entry->set_sensitive(state);
    high_w_kHz->set_sensitive(state);
  }
}

void DialogLameOptions::low_cutoff() {
  bool state = filter_lowpass_cutoff->get_active();

  filter_lowpass_cutoff_entry->set_sensitive(state);
  low_kHz->set_sensitive(state);
}

void DialogLameOptions::low_width() {
  bool state = filter_lowpass_width->get_active();

  filter_lowpass_width_entry->set_sensitive(state);
  low_w_kHz->set_sensitive(state);
}

void DialogLameOptions::high_cutoff() {
  bool state = filter_highpass_cutoff->get_active();

  filter_highpass_cutoff_entry->set_sensitive(state);
  high_kHz->set_sensitive(state);
}

void DialogLameOptions::high_width() {
  bool state = filter_highpass_width->get_active();

  filter_highpass_width_entry->set_sensitive(state);
  high_w_kHz->set_sensitive(state);
}

void DialogLameOptions::tonality() {
  bool state = filter_tonality->get_active();

  filter_tonality_entry->set_sensitive(state);
  tonality_kHz->set_sensitive(state);
}

vector<string> DialogLameOptions::get_lame_parameters() {
  vector<string> parameters;

  // disable progress report and VBR histogram
  parameters.push_back("-S");

  // bit reservoir parameter
  if (bit_reservoir->get_active()) parameters.push_back("--nores");

  // quality parameters
  if (q_high->get_active()) parameters.push_back("-h");
  if (q_low->get_active()) parameters.push_back("-f");

  if (!filter_all_freqs->get_active()) {

    // filter is not disabled
    if (filter_lowpass_cutoff->get_active()) {
      parameters.push_back("--lowpass");
      parameters.push_back(filter_lowpass_cutoff_entry->get_text());
    }

    if (filter_lowpass_width->get_active()) {
      parameters.push_back("--lowpass-width");
      parameters.push_back(filter_lowpass_width_entry->get_text());
    }

    if (filter_highpass_cutoff->get_active()) {
      parameters.push_back("--highpass");
      parameters.push_back(filter_highpass_cutoff_entry->get_text());
    }

    if (filter_highpass_width->get_active()) {
      parameters.push_back("--highpass-width");
      parameters.push_back(filter_highpass_width_entry->get_text());
    }
  }
  else {
    parameters.push_back("-k");
  }

  if (filter_tonality->get_active()) {
    parameters.push_back("--cwlimit");
    parameters.push_back(filter_tonality_entry->get_text());
  }


#ifdef DEBUG_OUTPUT
  cerr << "Lame options: ";
  copy(parameters.begin(), parameters.end(), ostream_iterator<string>(cerr, " "));
  cerr << endl;
#endif

  return parameters;
}
