/***************************************************************************
                 dialogoptions.cpp  -  Quickedit options dialog
                ------------------------------------------------
    begin                : Fri Apr 20 2001
    copyright            : (C) 2001 by Ladislav Slezk
    email                : slezak@penguin.cz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "dialogoptions.h"

#include <libgnome/gnome-i18n.h>


DialogOptions::DialogOptions() : Gtk::Window(GTK_WINDOW_DIALOG) {
  set_modal(true);
  set_title(_("Quickedit options"));
  set_policy(false, false, false);    // set fixed size of window

  Gtk::Label *fglabel = manage(new Gtk::Label(_("Foreground color")));
  fgpicker = manage(new Gnome::ColorPicker);
  fgpicker->set_title(_("Select foreground color"));
  Gtk::Label *bglabel = manage(new Gtk::Label(_("Background color")));
  bgpicker = manage(new Gnome::ColorPicker);
  bgpicker->set_title(_("Select background color"));

  Gtk::Table *table = manage(new Gtk::Table(2, 2, false));
  table->set_border_width(3);
  table->set_row_spacings(7);
  table->set_col_spacings(5);
  table->attach(*fglabel, 0, 1, 0, 1, 0, 0, 0, 0);
  table->attach(*bglabel, 0, 1, 1, 2, 0, 0, 0, 0);
  table->attach(*bgpicker, 1, 2, 1, 2, 0, 0, 0, 0);
  table->attach(*fgpicker, 1, 2, 0, 1, 0, 0, 5, 0);

  Gtk::Frame *frcolor = manage(new Gtk::Frame(_("Colors")));
  frcolor->add(*table);

  use_wheel = manage(new Gtk::CheckButton(_("Use wheel for zooming"), 0.0));
  use_wheel->set_border_width(3);

  Gtk::Frame *frmouse = manage(new Gtk::Frame(_("Mouse")));
  frmouse->add(*use_wheel);


  Gtk::Label *end_label = manage(new Gtk::Label(_("Play time")));
  end_time = manage(new Gtk::Entry);
  Gtk::Label *end_label_ms = manage(new Gtk::Label("ms"));

  Gtk::HBox *hbox_end = manage(new Gtk::HBox(false, 3));
  hbox_end->pack_start(*end_label, false, false);
  hbox_end->pack_start(*end_time, true, true);
  hbox_end->pack_start(*end_label_ms, false, false);
  hbox_end->set_border_width(5);

  Gtk::Frame *fr_end = manage(new Gtk::Frame(_("Play begin/end of selection")));
  fr_end->add(*hbox_end);


  Gtk::VBox *vbox_interface = manage(new Gtk::VBox(false, 3));
  vbox_interface->pack_start(*frcolor, false, false);
  vbox_interface->pack_start(*frmouse, false, false);
  vbox_interface->pack_start(*fr_end, false, false);
  vbox_interface->set_border_width(5);


  Gtk::Label *label_mpg123 = manage(new Gtk::Label("mpg123"));
  label_mpg123->set_usize(60, -1);
  label_mpg123->set_alignment(0.0, 0.5);
  Gtk::Label *label_lame = manage(new Gtk::Label("lame"));
  label_lame->set_usize(60, -1);
  label_lame->set_alignment(0.0, 0.5);
  Gtk::Label *label_sox = manage(new Gtk::Label("sox"));
  label_sox->set_usize(60, -1);
  label_sox->set_alignment(0.0, 0.5);

  string history_mpg123;
  entry_mpg123 = manage(new Gnome::FileEntry(history_mpg123, _("Set mpg123 executable")));
  entry_mpg123->set_modal(true);
  entry_mpg123->set_border_width(5);
  gentry_mpg123 = dynamic_cast<Gnome::Entry *>(entry_mpg123->gnome_entry());
  gentry_mpg123->set_history_id("mpg123_history");
  gentry_mpg123->load_history();

  string history_lame;
  entry_lame = manage(new Gnome::FileEntry(history_lame, _("Set lame executable")));
  entry_lame->set_modal(true);
  entry_lame->set_border_width(5);
  gentry_lame = dynamic_cast<Gnome::Entry *>(entry_lame->gnome_entry());
  gentry_lame->set_history_id("lame_history");
  gentry_lame->load_history();

  string history_sox;
  entry_sox = manage(new Gnome::FileEntry(history_sox, _("Set sox executable")));
  entry_sox->set_modal(true);
  entry_sox->set_border_width(5);
  gentry_sox = dynamic_cast<Gnome::Entry *>(entry_sox->gnome_entry());
  gentry_sox->set_history_id("sox_history");
  gentry_sox->load_history();

  Gtk::HBox *hbox_mpg123 = manage(new Gtk::HBox(false, 3));
  hbox_mpg123->pack_start(*label_mpg123, false, false);
  hbox_mpg123->pack_start(*entry_mpg123, true, true);
  hbox_mpg123->set_border_width(5);
  Gtk::HBox *hbox_lame = manage(new Gtk::HBox(false, 3));
  hbox_lame->pack_start(*label_lame, false, false);
  hbox_lame->pack_start(*entry_lame, true, true);
  hbox_lame->set_border_width(5);
  Gtk::HBox *hbox_sox = manage(new Gtk::HBox(false, 3));
  hbox_sox->pack_start(*label_sox, false, false);
  hbox_sox->pack_start(*entry_sox, true, true);
  hbox_sox->set_border_width(5);

  Gtk::Frame *fr_mpg123 = manage(new Gtk::Frame(_("MPEG decoder")));
  fr_mpg123->add(*hbox_mpg123);
  Gtk::Frame *fr_lame = manage(new Gtk::Frame(_("MPEG encoder")));
  fr_lame->add(*hbox_lame);
  Gtk::Frame *fr_sox = manage(new Gtk::Frame(_("Conversion program")));
  fr_sox->add(*hbox_sox);

  Gtk::VBox *vbox_programs = manage(new Gtk::VBox(false, 3));
  vbox_programs->pack_start(*fr_mpg123, false, false);
  vbox_programs->pack_start(*fr_lame, false, false);
  vbox_programs->pack_start(*fr_sox, false, false);
  vbox_programs->set_border_width(5);


  oss_file = manage(new Gtk::Entry());
  Gtk::Label *oss_label = manage(new Gtk::Label(_("Device file")));
  Gtk::HBox *hbox_oss = manage(new Gtk::HBox(false, 3));
  hbox_oss->pack_start(*oss_label, false, false);
  hbox_oss->pack_start(*oss_file, true, true);
  hbox_oss->set_border_width(5);
  Gtk::Frame *fr_oss = manage(new Gtk::Frame(_("Open Sound System")));
  fr_oss->add(*hbox_oss);

  Gtk::VBox *vbox_audio = manage(new Gtk::VBox(false, 3));
  vbox_audio->pack_start(*fr_oss, false, false);
  vbox_audio->set_border_width(5);

/////////
  Gtk::Tooltips *tips = manage(new Gtk::Tooltips);

  mp3_export_always_process = manage(new Gtk::CheckButton(_("Always process MP3 block by encoder"), 0.0));
  mp3_export_always_process->clicked.connect(slot(this, &DialogOptions::no_copy));
  tips->set_tip(*mp3_export_always_process, _("If checked all MP3 blocks will be reencoded"));

  mp3_export_strictequal = manage(new Gtk::CheckButton(_("Copy only if parameters are strict equal"), 0.0));
  tips->set_tip(*mp3_export_strictequal, _("Copy exported data only if all MP3 parametrs are equal"));

  Gtk::VBox *vbox_export_mp3 = manage(new Gtk::VBox(false, 3));
  vbox_export_mp3->pack_start(*mp3_export_always_process, false, false);
  vbox_export_mp3->pack_start(*mp3_export_strictequal, false, false);
  vbox_export_mp3->set_border_width(5);

  Gtk::Frame *fr_mp3_export = manage(new Gtk::Frame(_("Export MP3 blocks options")));
  fr_mp3_export->add(*vbox_export_mp3);

  Gtk::VBox *vbox_export = manage(new Gtk::VBox(false, 3));
  vbox_export->pack_start(*fr_mp3_export, false, false);
  vbox_export->set_border_width(5);
////////

  Gtk::Notebook *notebook = manage(new Gtk::Notebook);
  notebook->set_show_tabs(true);
  notebook->set_scrollable(true);
  notebook->set_tab_border(3);

  Gtk::Label *labelint = manage(new Gtk::Label(_("Interface")));
  notebook->append_page(*vbox_interface, *labelint);
  Gtk::Label *labelprg = manage(new Gtk::Label(_("Programs")));
  notebook->append_page(*vbox_programs, *labelprg);
  Gtk::Label *labelaudio = manage(new Gtk::Label(_("Audio device")));
  notebook->append_page(*vbox_audio, *labelaudio);
  Gtk::Label *labelexport = manage(new Gtk::Label(_("Export options")));
  notebook->append_page(*vbox_export, *labelexport);

  Gnome::StockButton *bok = manage(new Gnome::StockButton(GNOME_STOCK_BUTTON_OK));
  bok->clicked.connect(bind(slot(this, &DialogOptions::button_pressed), true));

  Gnome::StockButton *bcancel = manage(new Gnome::StockButton(GNOME_STOCK_BUTTON_CANCEL));
  bcancel->clicked.connect(bind(slot(this, &DialogOptions::button_pressed), false));

  Gtk::Button *defaults = manage(new Gtk::Button(_("Set defaults")));
  defaults->clicked.connect(slot(this, &DialogOptions::set_defaults));

  Gtk::HButtonBox *hbbox1 = manage(new Gtk::HButtonBox);
  hbbox1->add(*bok);
  hbbox1->add(*bcancel);
  hbbox1->add(*defaults);
  hbbox1->set_layout(GTK_BUTTONBOX_SPREAD);

  Gtk::VBox *vbox1 = manage(new Gtk::VBox(false, 5));
  vbox1->set_border_width(3);
  vbox1->pack_start(*notebook, false, false, 3);
  vbox1->pack_start(*hbbox1, true, false, 3);

  add(*vbox1);

  show_all();
}

DialogOptions::~DialogOptions() {
}

void DialogOptions::button_pressed(bool ok) {
  hide();

  if (ok) {
    gentry_mpg123->save_history();
    gentry_lame->save_history();
    gentry_sox->save_history();

    ok_pressed.emit();
  }
}

void DialogOptions::set_defaults() {
  // default colors - foreground: green
  fgpicker->set(0.0, 1.0, 0.0, 0.0);
  // background: black
  bgpicker->set(0.0, 0.0, 0.0, 0.0);

  // use wheel
  use_wheel->set_active(false);

  // set default end time
 	end_time->set_text("3000");

  // set default path to program name only
  gentry_mpg123->get_entry()->set_text("mpg123");
  gentry_lame->get_entry()->set_text("lame");
  gentry_sox->get_entry()->set_text("sox");

  //set default OSS device file
  oss_file->set_text("/dev/dsp");

  // set default MP3 export options
  mp3_export_always_process->set_active(true);
  mp3_export_strictequal->set_active(true);
  no_copy();

}

Gdk_Color DialogOptions::get_foreground_color() {
  Gdk_Color tmp_color;
  gushort r, g, b, a;

  fgpicker->get(r, g, b, a);
  tmp_color.set_rgb(r, g, b);

  return tmp_color;
}

Gdk_Color DialogOptions::get_background_color() {
  Gdk_Color tmp_color;
  gushort r, g, b, a;

  bgpicker->get(r, g, b, a);
  tmp_color.set_rgb(r, g, b);

  return tmp_color;
}
  	
void DialogOptions::set_foreground_color(Gdk_Color c) {
  gushort r, g, b, a;

  r = c.get_red();
  g = c.get_green();
  b = c.get_blue();
  a = 0;

  fgpicker->set(r, g, b, a);
}

void DialogOptions::set_background_color(Gdk_Color c) {
  gushort r, g, b, a;

  r = c.get_red();
  g = c.get_green();
  b = c.get_blue();
  a = 0;

  bgpicker->set(r, g, b, a);
}

int DialogOptions::delete_event_impl(GdkEventAny* e) {
  hide();

  return true;            // disable destroying window by WM
}

void DialogOptions::no_copy() {
  mp3_export_strictequal->set_sensitive(!mp3_export_always_process->get_active());
}
