/***************************************************************************
                 dialogresize.cpp  -  resize block dialog
                ------------------------------------------
    begin                : Sat Sep 30 2000
    copyright            : (C) 2000 by Ladislav Slezk
    email                : slezak@penguin.cz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "dialogresize.h"
#include "utils.h"

#include <gnome--/app.h>
#include <libgnome/gnome-i18n.h>


DialogResize::DialogResize(SoundBlock* sblock) : Gtk::Window(GTK_WINDOW_DIALOG) {
  sb = sblock;
  set_modal(true);
  set_border_width(5);
  set_title(_("Resize block"));
  set_policy(false, true, false);    // allow grow

  Gtk::VBox *vbox1 = manage(new Gtk::VBox(false, 5));

  Gtk::Frame *fbox1 = manage(new Gtk::Frame(_("Start of block")));
  Gtk::Frame *fbox2 = manage(new Gtk::Frame(_("End of block")));

  vbox1->pack_start(*fbox1, false, false, 0);
  vbox1->pack_start(*fbox2, false, false, 0);

  Gtk::HButtonBox *hbbox1 = manage(new Gtk::HButtonBox());
  vbox1->pack_start(*hbbox1, true, false, 0);

//  bok = manage(new Gtk::Button(_("Ok")));
  bok = manage(new Gnome::StockButton(GNOME_STOCK_BUTTON_OK));
  bok->clicked.connect(SigC::slot(this, &DialogResize::ok_pressed));

//  bcancel = manage(new Gtk::Button(_("Cancel")));
  bcancel = manage(new Gnome::StockButton(GNOME_STOCK_BUTTON_CANCEL));
  bcancel->clicked.connect(destroy.slot());

  hbbox1->add(*bok);
  hbbox1->add(*bcancel);
  hbbox1->set_layout(GTK_BUTTONBOX_SPREAD);


  Gtk::VBox *vbox2 = manage(new Gtk::VBox(false, 5));
  Gtk::HBox *hbox1 = manage(new Gtk::HBox(false, 5));
  Gtk::HBox *hbox2 = manage(new Gtk::HBox(false, 5));

  Gtk::Label *l1 = manage(new Gtk::Label(_("Sample: ")));
  hbox2->pack_start(*l1, false, false, 5);

  Gtk::Label *l2 = manage(new Gtk::Label(_("Time: ")));
  t1 = manage(new Gtk::Label);
  hbox1->pack_start(*l2, false, false, 5);
  hbox1->pack_start(*t1, false, false, 5);

  a1 = manage(new Gtk::Adjustment (sblock->get_begin_sample_offset(), 1.0, sblock->max_samples(), 1.0, 5.0, 0.0));
  Gtk::HScale *hscale1 = manage(new Gtk::HScale(*a1));
  a1->value_changed.connect(SigC::slot(this, &DialogResize::hscale_begin_changed));
  hscale1->set_digits(0);
  hbox2->pack_start(*hscale1, true, true, 5);


  Gtk::VBox *vbox22 = manage(new Gtk::VBox(false, 5));
  Gtk::HBox *hbox21 = manage(new Gtk::HBox(false, 5));
  Gtk::HBox *hbox22 = manage(new Gtk::HBox(false, 5));

  Gtk::Label *l21 = manage(new Gtk::Label(_("Time: ")));
  hbox21->pack_start(*l21, false, false, 5);
  t2 = manage(new Gtk::Label);
  hbox21->pack_start(*t2, false, false, 5);

  Gtk::Label *l22 = manage(new Gtk::Label(_("Sample: ")));
  hbox22->pack_start(*l22, false, false, 5);

  a2 = manage(new Gtk::Adjustment (sblock->get_end_sample_offset(), 1.0, sblock->max_samples(), 1.0, 5.0, 0.0));
  Gtk::HScale *hscale2 = manage(new Gtk::HScale(*a2));
  a2->value_changed.connect(SigC::slot(this, &DialogResize::hscale_end_changed));
  hscale2->set_digits(0);
  hbox22->pack_start(*hscale2, true, true, 5);


  // set begin time offset
  int Bms = static_cast<int>(static_cast<double>(sblock->get_begin_sample_offset()) / sblock->get_frequency() * 1e3);
  t1->set_text(Utils::ms2str(Bms));

  // set end time offset
  int Ems = static_cast<int>(static_cast<double>(sblock->get_end_sample_offset()) / sblock->get_frequency() * 1e3);
  t2->set_text(Utils::ms2str(Ems));


  vbox2->pack_start(*hbox1, false, false, 5);
  vbox2->pack_start(*hbox2, false, false, 5);
  vbox2->set_border_width(5);
  fbox1->add(*vbox2);
  fbox1->set_border_width(3);

  Gtk::VBox *vbox3 = manage(new Gtk::VBox(false, 5));
  vbox3->pack_start(*hbox21, false, false, 5);
  vbox3->pack_start(*hbox22, false, false, 5);
  vbox3->set_border_width(5);
  fbox2->add(*vbox3);
  fbox2->set_border_width(3);


  add(*vbox1);
  show_all();
}

DialogResize::~DialogResize() {
}

void DialogResize::ok_pressed() {
  sb->set_begin_sample_offset(static_cast<int>(a1->get_value()));
  sb->set_end_sample_offset(static_cast<int>(a2->get_value()));
  destroy();
}

void DialogResize::hscale_begin_changed() {
  int Bms = int (a1->get_value() / sb->get_frequency() * 1e3);
  t1->set_text(Utils::ms2str(Bms));
}

void DialogResize::hscale_end_changed() {
  int Ems = int (a2->get_value() / sb->get_frequency() * 1e3);
  t2->set_text(Utils::ms2str(Ems));
}
