/***************************************************************************
             soundblock.cpp  -  base class for all sound formats
            -----------------------------------------------------
    begin                : Wed Feb 2 2000
    copyright            : (C) 2000 by Ladislav Slezk
    email                : slezak@penguin.cz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "soundblock.h"

#include <gdk--/gc.h>

SoundBlock::SoundBlock(){
  changed = false;
  assigned = false;
  playing = false;
	paused = false;
	saveScannedData = true;
}

SoundBlock::~SoundBlock(){
}

SoundBlock::SoundBlock(const SoundBlock &src) {
  blockName = src.blockName;
  freq = src.freq;
  channels = src.channels;

  changed = src.changed;

  fileName = src.fileName;
	fileDateTime = src.fileDateTime;

  begin = src.begin;
  end = src.end;
  beginMax = src.beginMax;
  endMax = src.endMax;

 	sdev = src.sdev;

  assigned = src.assigned;
  playing = false;
  paused = false;

  scannedData = src.scannedData;
	saveScannedData = src.saveScannedData;
}

void SoundBlock::display(Gdk_Pixmap *pix, const Gdk_GC &gc, int offset, int height, int width) {
  int index = 0;
  int scanLen = static_cast<int>(static_cast<double>(scannedData.size()) * num_samples() / max_samples());
  int val;
  double step = static_cast<double>(scanLen) / width;
  int yoffset = (height / 2) / channels;
  double maxvalue = 1 << (16 - 1);	// 16 bits

  if (scanLen == 0) return;

  int scanOff = static_cast<int>(static_cast<double>(get_begin_sample_offset()) * scannedData.size() / max_samples());

  for (int i = 0; i < width;  i++) {
 		index = static_cast<int>(i*step) + scanOff;
 		
 		if (channels == 2 && index % 2 != 0) index++;	// index must be even on stereo sound

		val = static_cast<int>((scannedData[index] / maxvalue) * yoffset);
		
  	pix->draw_line(gc, offset + i, yoffset, offset + i, yoffset + val);
  	pix->draw_line(gc, offset + i, yoffset, offset + i, yoffset - val);

		if (channels == 2) {
			index++;
  		val = int ((scannedData[index] / maxvalue) * height / 4);
  		
  		pix->draw_line(gc, offset + i, static_cast<int>(0.75*height), offset + i, static_cast<int>(0.75*height + val));
	  	pix->draw_line(gc, offset + i, static_cast<int>(0.75*height), offset + i, static_cast<int>(0.75*height - val));
    }
  }

}

