/***************************************************************************
              subprocess.h  -  execute command on background
             ------------------------------------------------
    begin                : Tue Oct 3 2000
    copyright            : (C) 2000 by Ladislav Slezk
    email                : slezak@penguin.cz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SUBPROCESS_H
#define SUBPROCESS_H

#include <sigc++/signal_system.h>
#include <gtk--/main.h>
#include <unistd.h>
#include <signal.h>

#include <string>
#include <vector>
#include <map>

using namespace std;

/**
  *@author Ladislav Slezk
  */

class SubProcess : public SigC::Object {

  public:

	  SubProcess();
  	~SubProcess();
  	
  	bool start(string command, const vector <string> &args);    // start subprocess
  	bool kill_process(int sig = SIGTERM);                       // send signal to subprocess
  	
  	void write_stdin(const char *, int);                    // write data to stdin
  	
  	pid_t get_pid() {return pid;}                           // get pid of subprocess
  	bool running() {return run;}                            // is subprocess running?
  	void close_stdin();                                     // close stdin

  private:

    pid_t pid;
    int stdout_pipe[2],stderr_pipe[2],stdin_pipe[2];        // pipe descriptors for comunication
    bool run;

    const int buffer_size;
    char *buffer_out, *buffer_err, *buffer_in;
    char *pos_out, *pos_err, *pos_in;
    int std_out_len, std_err_len, std_in_len;
    int buffer_in_size;
    bool close_in;

    SigC::Connection c_in, c_out, c_err;

    void canReadStdOut(int, GdkInputCondition);
    void canReadStdErr(int, GdkInputCondition);
    void canWriteStdIn(int, GdkInputCondition);

    void exited(int);

    static map<pid_t, SubProcess*> pidmap;            // map pids to SubProcess
    static bool handler_installed;                    // is signal handler installed?

    static int get_status();
    static void signal_handler(int s);                // signal handler

    bool signaled_exit;
  	
  public:

		SigC::Signal1<void, int> process_exited;
		SigC::Signal0<void> data_written;
		SigC::Signal2<void, char *, int> stdout_data;
		SigC::Signal2<void, char *, int> stderr_data;
  	
};

#endif
