/***************************************************************************
                      view.cpp  -  waveform widget
                     ------------------------------
    begin                : Sun Jul 23 2000
    copyright            : (C) 2000 by Ladislav Slezk
    email                : slezak@penguin.cz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "view.h"

View::View() {
  set_events(GDK_EXPOSURE_MASK | GDK_BUTTON_PRESS_MASK | GDK_POINTER_MOTION_MASK | GDK_BUTTON_RELEASE_MASK);

//  Gdk_Visual vis = get_visual();
//  Gdk_Visual visbest = vis.get_best(GDK_VISUAL_PSEUDO_COLOR);
//  set_visual(visbest);

  position_visible = false;
  selection_visible = false;

  fgcolor.parse("green");
  bgcolor.parse("black");
}

View::~View() {
}

/* Create a new backing pixmap of the appropriate size */
int View::configure_event_impl (GdkEventConfigure *) {
  window = get_window();

  pixmap.release();
  pixmap.create(get_window(),  width(), height());

//  bgc = get_style()->get_fg_gc(get_state());
  bgc = Gdk_GC(pixmap);
  get_colormap().alloc(bgcolor);
  bgc.set_foreground(bgcolor);
  pixmap.draw_rectangle(bgc, true, 0, 0, width(), height());

  fgc = Gdk_GC(pixmap);
//  Gdk_Color fcolor;
//  fcolor.set_rgb(0, 65535, 0);
//  get_colormap().alloc(fcolor);
  get_colormap().alloc(fgcolor);
  fgc.set_foreground(fgcolor);

	reconfigured.emit();
  return 1;
}

/* redraw the screen from the backing pixmap */
int View::expose_event_impl(GdkEventExpose *e) {
  window.draw_pixmap(fgc, pixmap, e->area.x, e->area.y, e->area.x, e->area.y, e->area.width, e->area.height);

  if (position_visible && pos > e->area.x && pos < e->area.x + e->area.width) {
    Gdk_GC linegc = Gdk_GC(pixmap);
    Gdk_Color linecolor;

    linecolor.set_rgb(65535, 65535, 65535);
    get_colormap().alloc(linecolor);
    linegc.set_foreground(linecolor);
    linegc.set_function(GDK_XOR);

    window.draw_line(linegc, pos, e->area.y, pos, e->area.y + e->area.height);
  }

  if (selection_visible && e->area.x < sel2 && e->area.x + e->area.width > sel1) {
    int x1 = (sel1 < e->area.x) ? e->area.x : sel1;
    int x2 = (sel2 > e->area.x + e->area.width) ? e->area.x + e->area.width : sel2;

    if (x2 < x1) {
      int tmp = x1;
      x1 = x2;
      x2 = tmp;
    }

    Gdk_GC linegc = Gdk_GC(pixmap);
    Gdk_Color linecolor;

    linecolor.set_rgb(65535, 65535, 65535);
    get_colormap().alloc(linecolor);
    linegc.set_foreground(linecolor);
    linegc.set_function(GDK_XOR);

    window.draw_rectangle(linegc, 1, x1, e->area.y, x2 - x1, e->area.height);
  }

  return false;
}

void View::clear() {
  pixmap.draw_rectangle(bgc, true, 0, 0, width(), height());
  position_visible = false;
  selection_visible = false;
}

/* Redraw the screen from the backing pixmap */
void View::refresh() {
  window.draw_pixmap(fgc, pixmap, 0, 0, 0, 0, width(), height());
  if (position_visible) draw_pos(pos);
  if (selection_visible) draw_selection(sel1, sel2);
}

void View::draw_pos(int x) {
  if (x < 0 || x > width()) return;
//  cerr << "View::draws_pos1 " << x << endl;

  if (position_visible) clear_pos();  // clear last position

  position_visible = true;
  pos = x;

  Gdk_GC linegc = Gdk_GC(pixmap);
  Gdk_Color linecolor;

  linecolor.set_rgb(65535, 65535, 65535);
  get_colormap().alloc(linecolor);
  linegc.set_foreground(linecolor);
  linegc.set_function(GDK_XOR);

  window.draw_line(linegc, x, 0, x, pixmap.height());
//  cerr << "View::draws_pos2 " << x << endl;
}

void View::draw_selection(int x1, int x2) {

  if (x2 < x1) {
    int tmp = x1;
    x1 = x2;
    x2 = tmp;
  }

//  cerr << "Draw selection parameters: " << x1 << ' ' << x2 << endl;
//  cerr << "Old selection: " << sel1 << ' ' << sel2 << endl;

  if (selection_visible && x2 == sel2 && x1 == sel1) return;

  Gdk_GC linegc = Gdk_GC(pixmap);
  Gdk_Color linecolor;

  linecolor.set_rgb(65535, 65535, 65535);
  get_colormap().alloc(linecolor);
  linegc.set_foreground(linecolor);
  linegc.set_function(GDK_XOR);

  // resizing selection  -- start changed
  if (selection_visible && x2 == sel2 && x1 != sel1) {
    x2 = sel1;
    sel1 = x1;

    if (x2 < x1) {
      int tmp = x1;
      x1 = x2;
      x2 = tmp;
    }

    window.draw_rectangle(linegc, 1, x1, 0, x2 - x1, pixmap.height());

//    cerr << "Start changed:" << endl;
//    cerr << "Selection:  x1:   " << x1 << "  x2:   " << x2 << endl;
//    cerr << "Selection:  sel1: " << x1 << "  sel2: " << x2 << endl;

    return;
  }

  // resizing selection  -- end changed
  if (selection_visible && x1 == sel1 && x2 != sel2) {
    x1 = sel2;
    sel2 = x2;

    if (x2 < x1) {
      int tmp = x1;
      x1 = x2;
      x2 = tmp;
    }

    window.draw_rectangle(linegc, 1, x1, 0, x2 - x1, pixmap.height());

//    cerr << "End changed:" << endl;
//    cerr << "Selection:  x1:   " << x1 << "  x2:   " << x2 << endl;
//    cerr << "Selection:  sel1: " << x1 << "  sel2: " << x2 << endl;

    return;
  }

//  cerr << "New selection started" << endl;

  clear_selection();

  selection_visible = true;
  sel1 = x1;
  sel2 = x2;

  window.draw_rectangle(linegc, 1, x1, 0, x2 - x1, pixmap.height());
}

void View::clear_pos() {
  if (position_visible) {
    position_visible = false;
    draw_pos(pos);
    position_visible = false;
  }
}

void View::clear_selection() {
  if (selection_visible) {
    selection_visible = false;
    draw_selection(sel1, sel2);
    selection_visible = false;
  }
}

void View::set_foreground_color(Gdk_Color c) {
  fgcolor = c;

  get_colormap().alloc(fgcolor);
  fgc.set_foreground(fgcolor);

	reconfigured.emit();
}

void View::set_background_color(Gdk_Color c) {
  bgcolor = c;

  get_colormap().alloc(bgcolor);
  bgc.set_foreground(bgcolor);

  // set background style color - better scrolling
  Gtk::Style *st = Gtk::Style::create();
  st->set_bg(GTK_STATE_NORMAL, bgcolor);
  set_style(*st);

  reconfigured.emit();
}
