/***************************************************************************
              wavblock.h  -  implementation of WAV sound block
             --------------------------------------------------
    begin                : Sat Jul 29 2000
    copyright            : (C) 2000 by Ladislav Slezk
    email                : slezak@penguin.cz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef WAVBLOCK_H
#define WAVBLOCK_H

#include "soundblock.h"
#include "utils.h"
#include "subprocess.h"
#include "dialogprogress.h"

#include <sigc++/thread.h>
#include <sigc++/signal_system.h>
#include <gtk--/main.h>


/**Block of sound - format WAV
  *@author Ladislav Slezk
  */

class WAVblock : public SoundBlock, public SigC::Threads::Thread {

  public:

  	WAVblock();
  	WAVblock(const WAVblock &src);
  	~WAVblock();
  	
  	virtual bool init_play();  	
  	virtual bool play_block(SoundDevice *sd, string mpg123_path, int from_ms = -1, int to_ms = -1);
  	virtual bool is_changed();
  	virtual bool scan_file(int, int);					
  	virtual int block_length();								
  	virtual bool valid();											
  	virtual string block_info();							
  	virtual bool assign_file(string);
  	virtual bool assigned_file();							
  	virtual int num_samples();									
  	virtual int max_samples();									
	  virtual int read_sample(int pos, int channel = 0);
  	virtual void stop_block();
  	virtual void pause_block();
  	virtual void unpause_block();
  	virtual string get_type();
		virtual bool save_block_to_stream(ofstream &, string);
    virtual bool load_block_from_stream(ifstream &);
  	virtual int get_begin_sample_offset();
  	virtual int get_end_sample_offset();
  	virtual void set_begin_sample_offset(int);
  	virtual void set_end_sample_offset(int);

  	virtual void export_WAV(ofstream &ostr, int chan, int freq, int bps, DialogProgress *dp,
                           string decoder_path, string conversion_path);
  	
		virtual void export_MP3(ofstream &ostr, string encoder_name, string encoder_path, int bitrate, int frequency,
		                     ChannelMode channel_mode, bool crc, bool priv, bool original,
		                     bool copyright, vector<string> VBR_options, vector<string> encoder_options,
		                     bool always_process, bool strict_equal, DialogProgress *dp);
  	
  private:

    gint16 bitsPerSample;
    int buff_size;
    char *buffer;
    bool exporting_WAV;

    int num;					    // number of buffers
    int rem;					    // reminder

    int playedMS;
    int lastMS;

    void *main(void *);		// thread function

    int timer_done();	
		SigC::Connection cn;
		
		SubProcess conversion_subprocess;
		ofstream *export_stream;
		DialogProgress *export_progress;
		int block_num;
		
		// slots for SubProcess	
    void converter_exited(int);        // sound converter exited
    void data_written();               //
    void received_data(char *, int);   // received data from stdout
    void received_err(char *, int);    // received data from stderr

};

#endif
